// LogList.h: interface for the CLogList class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_LOGLIST_INCLUDED_)
#define _LOGLIST_INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "LogDesc.h"
#include "Engine/Engine.h"

#undef DECL
#ifdef LOGMGR_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

//////////////////////////////////////////////////////////////////////////
// This class manages the list of log files for the system. 
// 
class DECL CLogList
{
public:
   // Constructor/destructor. 
   CLogList();
   ~CLogList();
   
   // Search for logs and start monitoring for device insertions/removals. 
   // Will also create a system log, if one doesn't exist. 
   bool Start();
   
   // Stop monitoring for device changes, and empty the list. 
   bool Stop();

   // Create log using path to hypothetical log. User is prompted for desired 
   // log properties. If parent window arg is NULL, the default mfc top is used. 
   CLogDesc* CreateLog(CWnd* pParentWnd, LPCTSTR pszName = NULL);
   
   // Create log using defined log descriptor object. A "creating log" dialog is 
   // shown, hence the need for a parent window (default mfc top is used, if NULL). 
   bool CreateLog(CWnd* pParentWnd, CLogDesc* pNewLog);
   
   // Save/restore setup. 
   BOOL WriteSetup(CXMLSetup& XMLSetup);
   BOOL ReadSetup(CXMLSetup& XMLSetup, CXMLToken& XMLToken);

   // For finding logs and detecting changes at runtime. 
   void FindLogs();
   void DetectDeviceChange(); 

   bool GetIdealLogSize(const CString& strPath, int& iSize);

   // Lock and unlock against list additions and deletions. 
   bool LockRead(DWORD dwWait) {return m_rwLock.LockRead(dwWait);};
   bool LockWrite(DWORD dwWait) {return m_rwLock.LockWrite(dwWait);};
   void UnlockRead() {m_rwLock.UnlockRead();};
   void UnlockWrite() {m_rwLock.UnlockWrite();};

   // These methods are for searching and iterating through the list. 
   // Users should lock and unlock the list when using them.
   int GetCount(){return List.GetCount();};
   POSITION GetHeadPosition(){return List.GetHeadPosition();};
   CLogDesc * GetNext(POSITION& pos){return List.GetNext(pos);};
   CLogDesc * GetHead(){return List.GetHead();};
   CLogDesc * GetAt(POSITION pos){return List.GetAt(pos);};
   CLogDesc * GetByName(LPCTSTR lpszName);
   CLogDesc * GetByPartialMatch(CString Name);
   POSITION Find(CLogDesc * pLog){return List.Find(pLog);};
   POSITION FindIndex(int nIndex){return List.FindIndex(nIndex);};
   
   // Methods for adding and deleting items. These are protected by ListLock.
   void Add(CLogDesc * pLog);
   POSITION AddHead(CLogDesc * pLog);
   void RemoveAt(POSITION pos){LockWrite(INFINITE); List.RemoveAt(pos); UnlockWrite();};
   void FlushAll();
   void FlushQuick();
   void (*pFinalStop)();

   // The "default log" is set by user using SetDefault(). GetDefault() will 
   // return default if still valid, otherwise will search for first non-system 
   // log to be default, or system log if no others found, or NULL if list empty. 
   CString m_strDefault;
   CLogDesc* GetDefault();
   void SetDefault(CLogDesc* pLog) {pLog ? m_strDefault = pLog->GetName() : m_strDefault.Empty();};
   void SetDefault(LPCTSTR lpszName) {m_strDefault = lpszName;};

   // Dumps log data out debug port.
   void Dump();

protected:
   void RemoveAll(){LockWrite(INFINITE); List.RemoveAll(); UnlockWrite();};
   void Empty();

private:
   CList<CLogDesc*, CLogDesc*> List;      // The list itself. 
   HANDLE m_hThread;                      // Handle to dev change monitoring thread.
   HANDLE m_hStopEvent;                   // Global "stop" event. 
   TRWLock m_rwLock;                      // Protect consistency of list. 
   CRITICAL_SECTION m_csFind;             // Protect against colliding FindLogs().
};

extern "C" DECL CLogList LogList;

#endif // !defined(_LOGLIST_INCLUDED_)
